<?php

if ( ! defined( 'ABSPATH' ) ) {
	exit;	// Exit if accessed directly
}

define( 'MYFREIGHT_STATUS_MAX_INTERVAL', 1800 );		// Tracking bulk updates - polling interval in seconds
define( 'MYFREIGHT_STATUS_GAP_INTERVAL', 600 );			// Tracking bulk updates - minimum gap between ad hoc order list and polled in seconds
define( 'MYFREIGHT_STATUS_LIST_INTERVAL', 120 );		// Tracking bulk updates - ad hoc order list view interval in seconds
define( 'MYFREIGHT_STATUS_MIN_INTERVAL', 30 );			// Tracking single updates - ad hoc single order view interval in seconds
define( 'MYFREIGHT_SEARCH_MIN_INTERVAL', 300 );			// Tracking bulk updates - search for con cumulative interval in seconds

define( 'MYFREIGHT_STATUS_MAX_PROCESSING', 100 );		// Maximum number of processing orders to update tracking/order status for
define( 'MYFREIGHT_STATUS_LIST_PROCESSING', 100 );		// Maximum number of processing orders to update tracking/order status for when order list button clicked
define( 'MYFREIGHT_STATUS_MAX_COMPLETED', 200 );		// Maximum number of completed orders to update tracking status for
define( 'MYFREIGHT_STATUS_LIST_COMPLETED', 50 );		// Maximum number of completed orders to update tracking status for when order list button clicked
define( 'MYFREIGHT_SEARCH_MAX_PEAK', 10 );			// Maximum number of searches in a single tracking update in business hours
define( 'MYFREIGHT_SEARCH_MAX_OFFPEAK', 100 );			// Maximum number of searches in a single tracking update outside business hours

/**
 * Check if WooCommerce is active
 */
if ( in_array( 'woocommerce/woocommerce.php', apply_filters( 'active_plugins', get_option( 'active_plugins' ) ) ) ) {

	/**
	 * Persistent class for order tracking and status update functions
	 */
	if ( ! class_exists( 'WC_Myfreight_Tracking' ) ) {
		class WC_Myfreight_Tracking {

			public function __construct() {

				$settings = get_option( 'woocommerce_myfreightshipping_settings' );

				// Synchronise orders with tracking data
				if ( $settings['auto_consigning'] == 'yes' && ( $settings['tracking_sync'] == 'yes' || $settings['despatch_sync'] == 'yes' ) ) {
					add_filter( 'cron_schedules', array( $this, 'mf_processing_update_recurring' ), 10, 1 );	// Set up short interval schedule
					add_action( 'mf_processing_update_hook', array( $this, 'mf_processing_update_exec' ) );		// Update recent order status and tracking status
					if ( !wp_next_scheduled( 'mf_processing_update_hook' ) ) {					// Activate short interval events
						wp_schedule_event( time() + MYFREIGHT_STATUS_MAX_INTERVAL, 'mf_processing_update_interval', 'mf_processing_update_hook' );
						register_deactivation_hook( __FILE__, array( $this, 'mf_processing_update_deactivate' ) );
					}
					if ( is_admin() ) {
						if ( $settings['despatch_sync'] == 'yes' ) {
							add_action( 'manage_posts_extra_tablenav', array( $this, 'admin_order_list_top_bar_button' ), 20, 1 );	// Tracking button on order list page
							add_action( 'restrict_manage_posts', array( $this, 'mf_order_list_button_update_tracking' ), 10, 1 );	// Update tracking on button click
						}
						add_action( 'load-post.php', array( $this, 'mf_order_edit_update_tracking' ), 10, 0 );	// Update tracking status on order detail page load
					}
				}
				$daily_event = false;
				if ( $settings['auto_consigning'] == 'yes' && $settings['tracking_sync'] == 'yes' ) {
					add_action( 'mf_completed_update_hook', array( $this, 'mf_completed_update_exec' ) );		// Update tracking status for older orders
					$daily_event = true;
				}
				if ( $settings['surcharges_auto_update'] == 'yes' ) {
					add_action( 'mf_completed_update_hook', array( $this, 'mf_surcharge_update_exec' ) );		// Update carrier surcharge list
					$daily_event = true;
				}
				add_action( 'mf_completed_update_hook', array( $this, 'mf_report_counters' ) );				// Record quote and consign counters
				$daily_event = true;

				if ( $daily_event ) {
					if ( !wp_next_scheduled( 'mf_completed_update_hook' ) ) {					// Activate daily events
						$timezone_string = get_option( 'timezone_string' );
						date_default_timezone_set( !empty($timezone_string) ? $timezone_string : MY_DEFAULT_TIMEZONE );
						$start_minute = sprintf( '%02d',  intval( get_home_url(), 36 ) % 60 );			// Calculate random persistent minute within hour
						wp_schedule_event( strtotime( 'tomorrow 03:' . $start_minute . ' am' ), 'daily', 'mf_completed_update_hook' );
						register_deactivation_hook( __FILE__, array( $this, 'mf_completed_update_deactivate' ) );
					}
				}
			}


			/**
			 * Schedule recurring cron task to update orders with status = processing.
			 */
			public function mf_processing_update_recurring( $schedules ) {
				$schedules['mf_processing_update_interval'] = array(
					'interval' => MYFREIGHT_STATUS_MAX_INTERVAL,
					'display'  => esc_html__( 'Every ' . (string)(MYFREIGHT_STATUS_MAX_INTERVAL / 60) . ' minutes' )
				);
				return $schedules;
			}


			/**
			 * Remove recurring cron task when plugin deactivated.
			 */
			public function mf_processing_update_deactivate() {
//				$timestamp = wp_next_scheduled( 'mf_processing_update_hook' );
//				wp_unschedule_event( $timestamp, 'mf_processing_update_hook' );

				wp_clear_scheduled_hook( 'mf_processing_update_hook' );
			}


			/**
			 * Remove recurring cron task when plugin deactivated.
			 */
			public function mf_completed_update_deactivate() {
				wp_clear_scheduled_hook( 'mf_completed_update_hook' );
			}


			/**
			 * Perform scheduled task to update processing orders shipping status.
			 */
			public function mf_processing_update_exec() {

				$timezone_string = get_option( 'timezone_string' );
				date_default_timezone_set( !empty($timezone_string) ? $timezone_string : MY_DEFAULT_TIMEZONE );

				$current_timestamp = time();
				$hour = (int)date( 'G', $current_timestamp );
				if ( $hour > 20 || $hour < 6 ) {	// No updates between 9pm and 6am
					return;
				}

				$last_update = get_option( 'myfreightshipping_last_con_update', '2020-01-01 12:00:00' );
				$last_update_timestamp = strtotime( $last_update );

				if ( $current_timestamp < ( $last_update_timestamp + MYFREIGHT_STATUS_GAP_INTERVAL ) ) {
					return;
				}

				$this->mf_processing_update();

				$current_update = date( 'Y-m-d H:i:s' );
				update_option( 'myfreightshipping_last_con_update', $current_update, false );
			}


			/**
			 * Update processing orders shipping status.
			 */
			private function mf_processing_update( $limit = MYFREIGHT_STATUS_MAX_PROCESSING ) {

				$query = array(
					'status'       => array( 'wc-processing', 'wc-dispatched' ),
					'date_created' => '>' . ( time() - MONTH_IN_SECONDS ),
					'limit'        => $limit,
					'orderby'      => 'date',
					'order'        => 'DESC'
				);
				$orders = wc_get_orders( $query );

				$this->mf_shipping_status_update( $orders );
			}


			/**
			 * Perform scheduled task to update recently completed orders shipping status.
			 */
			public function mf_completed_update_exec() {

				$timezone_string = get_option( 'timezone_string' );
				date_default_timezone_set( !empty($timezone_string) ? $timezone_string : MY_DEFAULT_TIMEZONE );

				$this->mf_completed_update();
			}


			/**
			 * Update recently completed orders shipping status.
			 */
			private function mf_completed_update( $limit = MYFREIGHT_STATUS_MAX_COMPLETED ) {

				$query = array(
					'status'       => array( 'wc-completed', 'wc-dispatched' ),
					'date_created' => '>' . ( time() - ( 3 * MONTH_IN_SECONDS ) ),
					'limit'        => $limit,
					'orderby'      => 'date',
					'order'        => 'DESC'
				);
				$orders = wc_get_orders( $query );

				$this->mf_shipping_status_update( $orders );
			}


			/**
			 * Add tracking update button to top of order list page
			 */
			public function admin_order_list_top_bar_button( $which ) {
				global $pagenow, $typenow;

				if ( $typenow != 'shop_order' || $pagenow != 'edit.php' || $which != 'top' ) {
					return;									// If not order list page
				}

				$button_html = '<div class="alignleft actions tracking-update">';
				$button_html .= '<button id="update_tracking" name="update_tracking" class="button" type="submit" value="yes" style="height:32px;" title="' . __( 'Refresh shipment tracking data for all orders. Use sparingly.', 'woocommerce' ) . '">';
				$button_html .= __( 'Update Tracking', 'woocommerce' ) . '</button>';
				$button_html .= '</div>';
				echo $button_html;
			}


			/**
			 * When button is clicked update tracking status for orders
			 */
			public function mf_order_list_button_update_tracking( $post_type ) {
				global $pagenow;

				if ( $post_type != 'shop_order' || $pagenow != 'edit.php' || empty($_GET['update_tracking']) || $_GET['update_tracking'] != 'yes' ) {
					return;
				}

				$timezone_string = get_option( 'timezone_string' );
				date_default_timezone_set( !empty($timezone_string) ? $timezone_string : MY_DEFAULT_TIMEZONE );

				$current_timestamp = time();
				$last_update = get_option( 'myfreightshipping_last_con_update', '2020-01-01 12:00:00' );
				$last_update_timestamp = strtotime( $last_update );

				if ( $current_timestamp < ( $last_update_timestamp + MYFREIGHT_STATUS_LIST_INTERVAL ) ) {
					return;
				}

				$query = array(
					'status'       => array( 'wc-processing', 'wc-completed', 'wc-dispatched' ),
					'date_created' => '>' . ( time() - MONTH_IN_SECONDS ),
					'limit'        => MYFREIGHT_STATUS_LIST_PROCESSING,
					'orderby'      => 'date',
					'order'        => 'DESC'
				);
				$orders = wc_get_orders( $query );

				$this->mf_shipping_status_update( $orders );

				$current_update = date( 'Y-m-d H:i:s' );
				update_option( 'myfreightshipping_last_con_update', $current_update, false );
			}


			/**
			 * Update order tracking status when administrator opens order details/edit page.
			 */
			public function mf_order_edit_update_tracking() {
				global $typenow;

				if ( $typenow != 'shop_order' || empty($_GET['action']) || $_GET['action'] != 'edit' ) {
					return;									// If not order detail/edit page
				}

				$timezone_string = get_option( 'timezone_string' );
				date_default_timezone_set( !empty($timezone_string) ? $timezone_string : MY_DEFAULT_TIMEZONE );

				$current_timestamp = time();
				$last_update = get_option( 'myfreightshipping_last_con_update', '2020-01-01 12:00:00' );
				$last_update_timestamp = strtotime( $last_update );

				if ( $current_timestamp < ( $last_update_timestamp + MYFREIGHT_STATUS_MIN_INTERVAL ) ) {
					date_default_timezone_set( 'UTC' );
					return;
				}

				$order = wc_get_order( $_GET['post'] );
				if ( empty($order) ) {
					date_default_timezone_set( 'UTC' );
					return;
				}
				if ( !$order->has_status( array( 'processing', 'completed', 'dispatched' ) ) ) {
					date_default_timezone_set( 'UTC' );
					return;
				}

				$this->mf_shipping_status_update( array( $order ) );

				date_default_timezone_set( 'UTC' );
			}


			private static $logger;		// Handle for debug logging

			/**
			 * Get order consignment tracking status and update order.
			 */
			private function mf_shipping_status_update( $orders ) {

				$settings = get_option( 'woocommerce_myfreightshipping_settings' );
				$account_type = $settings['account_type'];
				$despatch_sync = ( $settings['despatch_sync'] == 'yes' );

				static $undespatched = array(
					"quote",
					"unbooked",
					"open",
					"despatchable",
					"despatching",
					"despatch_processing",
					"despatch_failure",
					"failed_permanently",
					"permanently_failed",
					"cancelled"
				);

				static $intransit = array(
					"picked_up",
					"in_transit",
					"on_board_for_delivery",
					"partial_delivery",
					"failed_delivery"
				);

				static $delivered = array(
					"delivered",
					"delivered_with_pod"
				);

				static $completed = array(
					"delivered",
					"delivered_with_pod",
					"cancelled"
				);

				/* Initialise debug */
				if ( $settings['debug'] == 'yes' ) {
					$timezone_string = get_option( 'timezone_string' );
					date_default_timezone_set( !empty($timezone_string) ? $timezone_string : MY_DEFAULT_TIMEZONE );
					self::$logger = wc_get_logger();
				} else {
					self::$logger = null;
				}
				if ( !empty( self::$logger ) ) {
					self::$logger->debug( '==== Updating Tracking Status for ' . count($orders) . ' Orders ====', array( 'source' => 'Myfreight Shipping' ) );
				}

				/* Determine maximum searches for con in Myfreight */
				$con_searches_due = 1;
				if ( $account_type == 'Myparcel' && count( $orders ) > 1 ) {
					$timezone_string = get_option( 'timezone_string' );
					date_default_timezone_set( !empty($timezone_string) ? $timezone_string : MY_DEFAULT_TIMEZONE );

					$current_timestamp = time();
					$hour = (int)date( 'G', $current_timestamp );
					if ( $hour > 16 || $hour < 8 ) {		// Bulk searches between 5pm and 8am
						$max_con_searches = MYFREIGHT_SEARCH_MAX_OFFPEAK;
					} else {
						$max_con_searches = MYFREIGHT_SEARCH_MAX_PEAK;
					}
					$last_search = get_option( 'myfreightshipping_last_con_search', '2020-01-01 12:00:00' );
					$last_search_timestamp = strtotime( $last_search );
					$con_searches_due += (int)floor( ( $current_timestamp - $last_search_timestamp ) / MYFREIGHT_SEARCH_MIN_INTERVAL );
					$con_searches_due = min( $con_searches_due, $max_con_searches );

					$current_search = date( 'Y-m-d H:i:s', $current_timestamp );
					update_option( 'myfreightshipping_last_con_search', $current_search, false );
				}

				/* Retrieve tracking status */
				$this->open_myfreight_consignment_info();

				foreach ( $orders as $order ) {
					if ( empty($order) || !method_exists( $order, 'get_meta' ) ) {
						continue;
					}

					$consignments = $order->get_meta( 'consignment', false, 'edit' );
					if ( empty($consignments) ) {
						continue;
					}

					$existing_cons = array();
					$max_valid_con_id = 0;
					$max_despatched_con_id = 0;
					$max_booking_id = 0;
					$max_con_status = null;
					foreach ( $consignments as &$consignment ) {
						$con_meta = $consignment->get_data();
						$con_info = $con_meta['value'];
						if ( !empty($con_info) && $con_info['account_type'] == $account_type ) {
							if ( ( $order->has_status( array( 'processing', 'dispatched' ) ) && $con_info['status'] != 'cancelled' ) ||
							     ( $order->has_status( 'completed' ) && !in_array( $con_info['status'], $completed ) ) ) {

								if ( $con_info['account_type'] == 'Myparcel' && !empty($con_info['booking']) ) {
									$pending_booking = explode( ':', $con_info['booking'] );
									$order_id = !empty($pending_booking[0]) ? $pending_booking[0] : $order->get_id();
								} else {
									$order_id = $order->get_id();
								}

								$reg_consignment = $this->get_myfreight_consignment_info( $con_info['account_type'], $con_info['test_mode'], $con_info['id'], $order_id, $con_searches_due, $existing_cons );

								if ( !empty($reg_consignment) && !empty($reg_consignment['state']) && empty($reg_consignment['errors']) ) {
									// Update order consignment status
									$new_con_info = array(
										'account_type'  => $con_info['account_type'],
										'test_mode'     => $con_info['test_mode'],
										'id'            => !empty($reg_consignment['id']) ? $reg_consignment['id'] : $con_info['id'],
										'service'       => $reg_consignment['carrier_name'] . ' ' . $reg_consignment['service_name'],
										'status'        => $reg_consignment['state'],
//										'con_link'      => $reg_consignment['url'],
										'con_link'      => $con_info['con_link'],
										'tracking_link' => ( !empty($reg_consignment['public_tracking_url']) && strpos( $reg_consignment['public_tracking_url'], 'track' ) ) ? $reg_consignment['public_tracking_url'] : $con_info['tracking_link']
									);
									if ( $con_info['account_type'] == 'Myparcel' ) {
										$booking_id = substr( strstr( $reg_consignment['custom_reference'] . ' ', ' ', true ), 3 );
										$new_con_info['booking'] = $booking_id;
									} else {
										$booking_id = 0;
									}
									$consignment->__set( 'value', $new_con_info );		// Save updated consignment

									// If tracking status has changed to next stage
									$old_tracking_status = $con_info['status'];
									$new_tracking_status = $new_con_info['status'];
									if ( $new_tracking_status != $old_tracking_status ) {
										if ( $old_tracking_status == 'cancelled' ) {
											$old_tracking_stage = 'Cancelled';
										} else if ( in_array( $old_tracking_status, $intransit ) ) {
											$old_tracking_stage = 'In Transit';
										} else if ( in_array( $old_tracking_status, $delivered ) ) {
											$old_tracking_stage = 'Delivered';
										} else {
											$old_tracking_stage = 'Booked';
										}
										if ( $new_tracking_status == 'cancelled' ) {
											$new_tracking_stage = 'Cancelled';
										} else if ( in_array( $new_tracking_status, $intransit ) ) {
											$new_tracking_stage = 'In Transit';
										} else if ( in_array( $new_tracking_status, $delivered ) ) {
											$new_tracking_stage = 'Delivered';
										} else {
											$new_tracking_stage = 'Booked';
										}

										if ( ( $new_tracking_stage == 'In Transit' && $old_tracking_stage != 'In Transit' && $old_tracking_stage != 'Delivered' ) ||
										     ( $new_tracking_stage == 'In Transit' && strpos( $new_tracking_status, 'delivery' ) !== false ) ||
										     ( $new_tracking_stage == 'Delivered' && $old_tracking_stage != 'Delivered' ) ||
										     ( $new_tracking_stage == 'Cancelled' && $old_tracking_stage != 'Cancelled' ) ) {
											/*
											 * Action hook to notify of shipment tracking status changes.
											 * Triggers on significant shipment stage changes: In Transit, Delivered and Cancelled.
											 * Also triggers on significant status changes within the In Transit stage.
											 * Note: The WooCommerce Completed Order email contains the initial shipment tracking information.
											 */
											do_action( 'my_shipment_tracking', $order, $new_con_info, $old_tracking_stage, $new_tracking_stage, $old_tracking_status, $new_tracking_status );
										}
									}

									// Save most recent and most recently despatched consignment id's
									$max_valid_con_id = max( $max_valid_con_id, (int)$reg_consignment['id'] );
									if ( !empty($reg_consignment['state']) && !in_array( $reg_consignment['state'], $undespatched ) ) {
										$max_despatched_con_id = max( $max_despatched_con_id, (int)$reg_consignment['id'] );
										$max_booking_id = max( $max_booking_id, (int)$booking_id );
										$max_con_status = $reg_consignment['state'];
									}
								}
							}
						}
					}
					unset( $consignment );

					$order->save_meta_data();

					// If order's most recent consignment despatched and auto completion enabled, change order status to completed
					if ( $max_despatched_con_id > 0 && $max_despatched_con_id == $max_valid_con_id ) {
						if ( $despatch_sync && ( $order->has_status( 'processing' ) || ( $order->has_status( 'dispatched' ) && in_array( $max_con_status, array_merge( $intransit, $delivered ) ) ) ) ) {
							if ( $account_type == 'Myfreight' ) {
								$order_note = 'Consignment ' . (string)$max_despatched_con_id . ' despatched in ' . $account_type . '.';
							} else {
								$order_note = 'Booking ' . (string)$max_booking_id . ' despatched in ' . $account_type . '.';
							}
							$order->set_status( 'completed', $order_note );
							date_default_timezone_set( 'UTC' );
							$order->save();
						}
					}
				}

				$this->close_myfreight_consignment_info();
			}


			/**
			 * Retrieve order consignment data from Myfreight.
			 */
			private $ch = array();

			private function open_myfreight_consignment_info() {

				global $ch;

				$settings = get_option( 'woocommerce_myfreightshipping_settings' );

				if ( $settings['account_type'] == 'Myfreight' ) {
					$credentials = $settings['myfreight_username'] . ':' . $settings['myfreight_password'];
				} else {
					$credentials = $settings['myparcel_username'] . ':' . $settings['myparcel_password'];
				}

				$header = array(
					'Content-type: application/json',
					'Accept: application/json',
					'Authorization: Basic ' . base64_encode( $credentials )
				);

				$ch = curl_init();
				curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
				curl_setopt($ch, CURLOPT_HTTPAUTH, CURLAUTH_BASIC);
				curl_setopt($ch, CURLOPT_HTTPHEADER, $header);
				curl_setopt($ch, CURLOPT_HTTPGET, true);
				curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
				curl_setopt($ch, CURLOPT_ENCODING, '');
				curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, 10);
				curl_setopt($ch, CURLOPT_TIMEOUT, 20);
				curl_setopt($ch, CURLOPT_VERBOSE, false);
			}

			private function close_myfreight_consignment_info() {

				global $ch;

				curl_close($ch);
			}

			private function get_myfreight_consignment_info( $account_type, $test_mode, $con_id, $order_id, &$con_searches_due, &$existing_cons ) {

				global $ch;

				$query_string = null;

				if ( empty($con_id) ) {			// If consignment ID not yet known (i.e. Myparcel), search for consignment containing order number hash

					if ( $account_type == 'Myparcel' && $con_searches_due > 0 ) {
						$con_searches_due--;
						$settings = get_option( 'woocommerce_myfreightshipping_settings' );
						$sync_url = $settings['myparcel_sync_url'];
						$origin_url = parse_url( $sync_url );
						$origin_id = $origin_url['host'] . $origin_url['path'] . $order_id;

						if ( $test_mode == 'yes' ) {
							$url = 'https://uat.teamwilberforce.com/public/api/v1/consignments/search?query=' . md5( $origin_id );
						} else {
							$url = 'https://go.myfreight.com.au/public/api/v1/consignments/search?query=' . md5( $origin_id );
						}
						curl_setopt($ch, CURLOPT_URL, $url);

						$data = curl_exec($ch);			// Send search data

						if ( !curl_errno($ch) && !empty($data) ) {
							if ( $results = (array)json_decode( $data, true ) ) {

								if ( empty( $results['errors'] ) ) {

									if ( isset($results['consignments']) && count( $results['consignments'] ) >= 1 && count( $results['consignments'] ) <= 8 ) {	// If match(es) found
										foreach ( $results['consignments'] as $matched_con ) {
											if ( !in_array( (int)$matched_con, $existing_cons ) ) {
												$con_id = (int)$matched_con;				// Save matched consignment if new
												break;
											}
										}
									}
								} else {
									if ( !empty( self::$logger ) ) {
										self::$logger->debug( 'CONSIGNMENT SEARCH ERROR: ' . print_r( $results['errors'][0], true ), array( 'source' => 'Myfreight Shipping' ) );
									}
								}
							} else {
								if ( !empty( self::$logger ) ) {
									self::$logger->debug( 'CONSIGNMENT SEARCH DECODE ERROR: ' . print_r( $data, true ), array( 'source' => 'Myfreight Shipping' ) );
								}
							}
						} else if ( curl_errno($ch) ) {
							if ( !empty( self::$logger ) ) {
								self::$logger->debug( 'CONSIGNMENT SEARCH API ERROR: ' . print_r( curl_errno($ch), true ), array( 'source' => 'Myfreight Shipping' ) );
							}
						}
					}
				}

				if ( empty($con_id) ) {
					return false;
				}
				$existing_cons[] = (int)$con_id;

				if ( $test_mode == 'yes' ) {
					$url = 'https://uat.teamwilberforce.com/public/api/v1/consignments/' . $con_id;
				} else {
					$url = 'https://go.myfreight.com.au/public/api/v1/consignments/' . $con_id;
				}
				curl_setopt($ch, CURLOPT_URL, $url);

//				if ( !empty( self::$logger ) ) {
//					self::$logger->debug( 'CONSIGNMENT STATUS REQUEST SENT: ' . print_r( $con_id, true ), array( 'source' => 'Myfreight Shipping' ) );
//				}

				$data = curl_exec($ch);			// Send consignment data

//				if ( !empty( self::$logger ) ) {
//					self::$logger->debug( 'CONSIGNMENT ' . $con_id . ' STATUS RECEIVED: ' . print_r( $data, true ), array( 'source' => 'Myfreight Shipping' ) );
//				}

				if ( !curl_errno($ch) && !empty($data) ) {
					if ( $consignment = (array)json_decode( $data, true ) ) {

						if ( empty( $consignment['errors'] ) ) {
//							if ( !empty( self::$logger ) ) {
//								self::$logger->debug( 'CONSIGNMENT ' . $con_id . ' STATUS RESPONSE: ' . print_r( $consignment, true ), array( 'source' => 'Myfreight Shipping' ) );
//							}

							// Save order shipping status
							return $consignment;
						} else {
							if ( !empty( self::$logger ) ) {
								self::$logger->debug( 'CONSIGNMENT ' . $con_id . ' (ORDER #' . $order_id . ') STATUS ERROR: ' . print_r( $consignment['errors'][0], true ), array( 'source' => 'Myfreight Shipping' ) );
							}
						}
					} else {
						if ( !empty( self::$logger ) ) {
							self::$logger->debug( 'CONSIGNMENT ' . $con_id . ' STATUS DECODE ERROR: ' . print_r( $data, true ), array( 'source' => 'Myfreight Shipping' ) );
						}
					}
				} else if ( curl_errno($ch) ) {
					if ( !empty( self::$logger ) ) {
						self::$logger->debug( 'CONSIGNMENT ' . $con_id . ' STATUS API ERROR: ' . print_r( curl_errno($ch), true ), array( 'source' => 'Myfreight Shipping' ) );
					}
				}

				return false;
			}


			/**
			 * Update order tracking status manually from order admin order details/edit page.
			 */
			public function mf_shipping_manual_status_update( $order, $manual_status ) {

				$settings = get_option( 'woocommerce_myfreightshipping_settings' );
				$account_type = $settings['account_type'];
				$despatch_sync = ( $settings['despatch_sync'] == 'yes' );

				static $intransit = array(
					"picked_up",
					"in_transit",
					"on_board_for_delivery",
					"partial_delivery",
					"failed_delivery"
				);

				static $delivered = array(
					"delivered",
					"delivered_with_pod"
				);

				static $completed = array(
					"delivered",
					"delivered_with_pod",
					"cancelled"
				);

				/* Initialise debug */
				if ( $settings['debug'] == 'yes' ) {
					$timezone_string = get_option( 'timezone_string' );
					date_default_timezone_set( !empty($timezone_string) ? $timezone_string : MY_DEFAULT_TIMEZONE );
					self::$logger = wc_get_logger();
				} else {
					self::$logger = null;
				}
				if ( !empty( self::$logger ) ) {
					self::$logger->debug( '==== Updating Tracking Status manually for order ' . $order . ' to ' . $manual_status . ' ====', array( 'source' => 'Myfreight Shipping' ) );
				}

				if ( empty($order) || !method_exists( $order, 'get_meta' ) ) {
					return false;
				}

				$consignments = $order->get_meta( 'consignment', false, 'edit' );
				if ( empty($consignments) ) {
					return false;
				}

				$old_tracking_status = ( $account_type == 'Myfreight' ) ? 'open' : 'unbooked';
				$new_tracking_status = $manual_status;
				$tracking_found = false;

				foreach ( $consignments as &$consignment ) {
					$con_meta = $consignment->get_data();
					$con_info = $con_meta['value'];
					if ( !empty($con_info) && ( $con_info['account_type'] == 'Tracking' || !empty($con_info['tracking_link']) ) ) {

						$tracking_found = true;
						$old_tracking_status = $con_info['status'];

						if ( $con_info['status'] != $manual_status ) {
							if ( ( $order->has_status( array( 'processing', 'dispatched' ) ) && $con_info['status'] != 'cancelled' ) ||
							     ( $order->has_status( 'completed' ) && !in_array( $con_info['status'], $completed ) ) ) {

								// Update order consignment status
								$new_con_info = array(
									'account_type'  => $con_info['account_type'],
									'test_mode'     => $con_info['test_mode'],
									'id'            => $con_info['id'],
									'service'       => $account_type . ' Tracking',
									'status'        => $manual_status,
//									'con_link'      => $con_info['con_link'],
									'tracking_link' => $con_info['tracking_link']
//									'booking'       => $con_info['booking']
								);
								$consignment->__set( 'value', $new_con_info );
								$order->save_meta_data();
							}
						}
						break;
					}
				}
				unset( $consignment );

				if ( !$tracking_found ) {
					/* Create dummy consignment for manual tracking */
					$new_con_info = array(
						'account_type'  => 'Tracking',
						'test_mode'     => $settings['test_mode'],
						'id'            => 0,
						'service'       => $account_type . ' Tracking',
						'status'        => $manual_status,
//						'con_link'      => null,
						'tracking_link' => null
//						'booking'       => null
					);

					if ( method_exists( $order, 'add_meta_data' ) ) {
						$order->add_meta_data( 'consignment', $new_con_info );
						$order->save_meta_data();
					}
				} else if ( empty($new_con_info) ) {
					$new_con_info = $con_info;
				}

				// If tracking status has changed to next stage
				if ( $new_tracking_status != $old_tracking_status ) {
					if ( $old_tracking_status == 'cancelled' ) {
						$old_tracking_stage = 'Cancelled';
					} else if ( in_array( $old_tracking_status, $intransit ) ) {
						$old_tracking_stage = 'In Transit';
					} else if ( in_array( $old_tracking_status, $delivered ) ) {
						$old_tracking_stage = 'Delivered';
					} else {
						$old_tracking_stage = 'Booked';
					}
					if ( $new_tracking_status == 'cancelled' ) {
						$new_tracking_stage = 'Cancelled';
					} else if ( in_array( $new_tracking_status, $intransit ) ) {
						$new_tracking_stage = 'In Transit';
					} else if ( in_array( $new_tracking_status, $delivered ) ) {
						$new_tracking_stage = 'Delivered';
					} else {
						$new_tracking_stage = 'Booked';
					}

					if ( ( $new_tracking_stage == 'In Transit' && $old_tracking_stage != 'In Transit' && $old_tracking_stage != 'Delivered' ) ||
					     ( $new_tracking_stage == 'In Transit' && strpos( $new_tracking_status, 'delivery' ) !== false ) ||
					     ( $new_tracking_stage == 'Delivered' && $old_tracking_stage != 'Delivered' ) ||
					     ( $new_tracking_stage == 'Cancelled' && $old_tracking_stage != 'Cancelled' ) ) {
						/*
						 * Action hook to notify of shipment tracking status changes.
						 * Triggers on significant shipment stage changes: In Transit, Delivered and Cancelled.
						 * Also triggers on significant status changes within the In Transit stage.
						 * Note: The WooCommerce Completed Order email contains the initial shipment tracking information.
						 */
						do_action( 'my_shipment_tracking', $order, $new_con_info, $old_tracking_stage, $new_tracking_stage, $old_tracking_status, $new_tracking_status );
					}

					$order->add_order_note( 'Tracking status manually updated to ' . $new_tracking_stage . '.', false, false );
					date_default_timezone_set( 'UTC' );
					$order->save();
				}

				// If manual tracking status despatched and auto completion enabled, change order status to completed
				if ( $despatch_sync && ( $order->has_status( 'processing' ) || ( $order->has_status( 'dispatched' ) && in_array( $manual_status, array_merge( $intransit, $delivered ) ) ) ) ) {
					$order_note = 'Tracking manually despatched.';

					$order->set_status( 'completed', $order_note );
					date_default_timezone_set( 'UTC' );
					$order->save();
				}

				return true;
			}


			/**
			 * Perform scheduled task to upload surcharge list and save in settings
			 */
			public function mf_surcharge_update_exec( $save_option = true ) {

				$settings = get_option( 'woocommerce_myfreightshipping_settings' );

				if ( $save_option && $settings['surcharges_auto_update'] != 'yes' ) {
					return false;
				}

				if ( $settings['account_type'] == 'Myfreight' ) {
					$surcharges_file_url = MYFREIGHT_SURCHARGES_URL;
				} else {
					$surcharges_file_url = MYPARCEL_SURCHARGES_URL;
				}

				$surcharges_json = file_get_contents( $surcharges_file_url );

				if ( empty($surcharges_json) || $surcharges_json[0] != '{' ) {
					return false;
				}

				if ( !$save_option ) {
					return $surcharges_json;
				} else {
					if ( $surcharges_json != htmlspecialchars_decode( $settings['surcharges'] ) ) {
						$settings['surcharges'] = $surcharges_json;
						update_option( 'woocommerce_myfreightshipping_settings', $settings, true );
						return true;
					}
				}
				return false;
			}


			/**
			 * Perform scheduled task to record counters
			 */
			public function mf_report_counters() {

				if ( date( 'w' ) != 0 ) {
					return;
				}

				$settings = get_option( 'woocommerce_myfreightshipping_settings' );

				$to = array(
					'admin@myparcel.com.au'
				);
				$headers = array(
					'From: ' . get_bloginfo( 'name' ) . ' <' . get_bloginfo( 'admin_email' ) . '>',
					'Content-Type: text/html; charset=UTF-8'
				);
				$subject = 'Quote and Consign Counters';

				$body = '<p>Usage counters for WooCommerce site:</p>';
				$body .= '<p>' . get_home_url() . '</p>';
				$body .= '<p>';
				foreach ( get_option( 'myfreightshipping_counters', array() ) as $counter_name => $value ) {
					$body .= ucfirst( str_replace( '_', ' ', $counter_name ) ) . ' = ' . $value . '<br>';
				}
				$body .= '</p>';
				$body .= '<p>' . ( $settings['account_type'] == 'Myparcel' ? 'Myparcel Account: ' . $settings['myparcel_username'] . ' (' . $settings['myparcel_member_username'] . ')' : 'Myfreight Account: ' . $settings['myfreight_username'] ) . '</p>';
				$body .= '<p>Plugin Version: ' . MYFREIGHT_SHIPPING_VERSION . '</p>';
				$body .= '<hr />';

				return wp_mail( $to, $subject, $body, $headers );
			}

		}
	}
	// Instantiate class
	new WC_Myfreight_Tracking();

}

